@@ -59,15 +59,21 @@ struct WaylandEGLConnection {
         std::call_once(s_onceFlag,
             [] {
                 s_connection.display = wl_display_connect(nullptr);
-                if (!s_connection.display)
+                if (!s_connection.display) {
+                    g_warning("WaylandEGLConnection: Could not connect to Wayland Display");
                     return;
+                }
 
                 EGLDisplay eglDisplay = eglGetDisplay(s_connection.display);
-                if (eglDisplay == EGL_NO_DISPLAY)
+                if (eglDisplay == EGL_NO_DISPLAY) {
+                    g_warning("WaylandEGLConnection: No EGL Display available in this connection");
                     return;
+                }
 
-                if (!eglInitialize(eglDisplay, nullptr, nullptr) || !eglBindAPI(EGL_OPENGL_ES_API))
+                if (!eglInitialize(eglDisplay, nullptr, nullptr) || !eglBindAPI(EGL_OPENGL_ES_API)) {
+                    g_warning("WaylandEGLConnection: Failed to initialize and bind the EGL Display");
                     return;
+                }
 
                 s_connection.eglDisplay = eglDisplay;
                 wpe_fdo_initialize_for_egl_display(s_connection.eglDisplay);
@@ -512,11 +518,15 @@ WindowViewBackend::WindowViewBackend(uint32_t width, uint32_t height)
     m_initialSize.height = height;
 
     auto& connection = WaylandEGLConnection::singleton();
-    if (!connection.display)
+    if (!connection.display) {
+        g_warning("WindowViewBackend: No Wayland EGL connection available");
         return;
+    }
 
-    if (connection.eglDisplay == EGL_NO_DISPLAY || !initialize(connection.eglDisplay))
+    if (connection.eglDisplay == EGL_NO_DISPLAY || !initialize(connection.eglDisplay)) {
+        g_warning("WindowViewBackend: Could not initialize EGL display");
         return;
+    }
 
     {
         auto* registry = wl_display_get_registry(connection.display);
@@ -540,7 +550,7 @@ WindowViewBackend::WindowViewBackend(uint32_t width, uint32_t height)
         source.pfd.revents = 0;
         g_source_add_poll(&source.source, &source.pfd);
 
-        g_source_set_priority(&source.source, G_PRIORITY_HIGH + 30);
+        g_source_set_priority(&source.source, G_PRIORITY_DEFAULT);
         g_source_set_can_recurse(&source.source, TRUE);
         g_source_attach(&source.source, g_main_context_get_thread_default());
     }
@@ -563,11 +573,15 @@ WindowViewBackend::WindowViewBackend(uint32_t width, uint32_t height)
     auto createPlatformWindowSurface =
         reinterpret_cast<PFNEGLCREATEPLATFORMWINDOWSURFACEEXTPROC>(eglGetProcAddress("eglCreatePlatformWindowSurfaceEXT"));
     m_eglSurface = createPlatformWindowSurface(connection.eglDisplay, m_eglConfig, m_eglWindow, nullptr);
-    if (!m_eglSurface)
+    if (!m_eglSurface) {
+        g_warning("WindowViewBackend: Could not create EGL platform window surface");
         return;
+    }
 
-    if (!eglMakeCurrent(connection.eglDisplay, m_eglSurface, m_eglSurface, m_eglContext))
+    if (!eglMakeCurrent(connection.eglDisplay, m_eglSurface, m_eglSurface, m_eglContext)) {
+        g_warning("WindowViewBackend: Could not make EGL surface current");
         return;
+    }
 
     imageTargetTexture2DOES = reinterpret_cast<PFNGLEGLIMAGETARGETTEXTURE2DOESPROC>(eglGetProcAddress("glEGLImageTargetTexture2DOES"));
 
@@ -664,6 +678,11 @@ const struct wl_callback_listener WindowViewBackend::s_frameListener = {
     }
 };
 
+struct wpe_view_backend* WindowViewBackend::backend() const
+{
+    return m_exportable ? wpe_view_backend_exportable_fdo_get_view_backend(m_exportable) : nullptr;
+}
+
 void WindowViewBackend::createViewTexture()
 {
     glGenTextures(1, &m_viewTexture);
@@ -697,6 +716,79 @@ void WindowViewBackend::resize(uint32_t width, uint32_t height)
     createViewTexture();
 }
 
+bool WindowViewBackend::initialize(EGLDisplay eglDisplay)
+{
+    static const EGLint configAttributes[13] = {
+        EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
+        EGL_RED_SIZE, 1,
+        EGL_GREEN_SIZE, 1,
+        EGL_BLUE_SIZE, 1,
+        EGL_ALPHA_SIZE, 1,
+        EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
+        EGL_NONE
+    };
+
+    {
+        EGLint count = 0;
+        if (!eglGetConfigs(eglDisplay, nullptr, 0, &count) || count < 1)
+            return false;
+
+        EGLConfig* configs = g_new0(EGLConfig, count);
+        EGLint matched = 0;
+        if (eglChooseConfig(eglDisplay, configAttributes, configs, count, &matched) && !!matched)
+            m_eglConfig = configs[0];
+        g_free(configs);
+    }
+
+    static const EGLint contextAttributes[3] = {
+        EGL_CONTEXT_CLIENT_VERSION, 2,
+        EGL_NONE
+    };
+
+    m_eglContext = eglCreateContext(eglDisplay, m_eglConfig, EGL_NO_CONTEXT, contextAttributes);
+    if (!m_eglContext)
+        return false;
+
+    static struct wpe_view_backend_exportable_fdo_egl_client exportableClient = {
+        // export_egl_image
+        nullptr,
+        // export_fdo_egl_image
+        [](void* data, struct wpe_fdo_egl_exported_image* image)
+        {
+            static_cast<WindowViewBackend*>(data)->displayBuffer(image);
+        },
+#if WPE_FDO_CHECK_VERSION(1, 5, 0)
+        // export_shm_buffer
+        [](void* data, struct wpe_fdo_shm_exported_buffer* buffer)
+        {
+            static_cast<WindowViewBackend*>(data)->displayBuffer(buffer);
+        },
+        // padding
+        nullptr, nullptr
+#else
+        // padding
+        nullptr, nullptr, nullptr
+#endif
+
+    };
+    m_exportable = wpe_view_backend_exportable_fdo_egl_create(&exportableClient, this, m_width, m_height);
+
+    initializeAccessibility();
+
+    return true;
+}
+
+void WindowViewBackend::deinitialize(EGLDisplay eglDisplay)
+{
+    m_inputClient = nullptr;
+
+    if (m_eglContext)
+        eglDestroyContext(eglDisplay, m_eglContext);
+
+    if (m_exportable)
+        wpe_view_backend_exportable_fdo_destroy(m_exportable);
+}
+
 void WindowViewBackend::displayBuffer(struct wpe_fdo_egl_exported_image* image)
 {
     if (!m_eglContext)
